import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_payment/ocg_payment.dart';
import 'package:ocg_payment/src/blocs/method.dart';
import 'package:ocg_payment/src/metadata/colors.dart';
import 'package:ocg_payment/src/modals/method.dart';
import 'package:ocg_payment/src/widgets/bank.dart';
import 'package:ocg_payment/src/widgets/card.dart';
import 'package:ocg_payment/src/widgets/wallet.dart';

class PaymentMethodScreen extends StatefulWidget {
  @override
  _PaymentMethodScreenState createState() => _PaymentMethodScreenState();
}

class _PaymentMethodScreenState extends State<PaymentMethodScreen> {
  PaymentMethod _selected;
  @override
  void initState() {
    super.initState();
    _refresh();
  }

  Future<void> _refresh() async {
    BlocProvider.of<PaymentMethodListBloc>(context)
        .add(PaymentMethodListRefreshEvent());
  }

  @override
  Widget build(BuildContext context) {
    double _width = MediaQuery.of(context).size.width;
    return Scaffold(
      appBar: AppBar(
        centerTitle: false,
        title: Text(
          "Payment Method",
          style: TextStyle(
              color: MetaColors.primary,
              fontSize: 18,
              fontWeight: FontWeight.w500),
        ),
        elevation: 0,
      ),
      body: Padding(
        padding: EdgeInsets.all(_width * 0.05),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: BlocBuilder<PaymentMethodListBloc, PaymentMethodListState>(
                builder: (context, state) {
                  if (state is PaymentMethodListLoadedState) {
                    var methods = state.paymentMethods;
                    List<PaymentBank> banks = List<PaymentBank>.from(methods
                        .where((element) => element is PaymentBank)
                        .map((e) => e as PaymentBank)).toList();
                    List<PaymentCard> cards = List<PaymentCard>.from(methods
                        .where((element) => element is PaymentCard)
                        .map((e) => e as PaymentCard));
                    List<PaymentWallet> wallets = List<PaymentWallet>.from(
                        methods
                            .where((element) => element is PaymentWallet)
                            .map((e) => e as PaymentWallet));
                    return ListView(
                      physics: ClampingScrollPhysics(),
                      shrinkWrap: true,
                      children: <Widget>[
                        buildWalletsList(wallets),
                        buildBanksList(banks),
                        buildCardsList(cards)
                      ],
                    );
                  }
                  if (state is PaymentMethodListLoadingState) {
                    return Center(
                      child: AppProgressIndicator(),
                    );
                  }
                  if (state is PaymentMethodListErrorState) {
                    return Center(
                      child: AppErrorWidget(
                        error: state.error,
                        onRefresh: _refresh,
                      ),
                    );
                  }
                  return Container();
                },
              ),
            ),
            SizedBox(
              height: 24,
            ),
            // Container(
            //   width: _width * 0.9,
            //   child: OutlineButtonBlueBorder(
            //     text: "Add a new account",
            //     padding: EdgeInsets.symmetric(vertical: 10),
            //     fontSize: 18,
            //     onPressed: PaymentHandler.of(context).addBankAccount,
            //   ),
            // ),
            // SizedBox(
            //   height: 16,
            // ),
            Container(
              width: _width * 0.9,
              child: RaisedButton(
                color: MetaColors.logoColor,
                padding: EdgeInsets.symmetric(vertical: 12),
                shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(6)),
                child: Text(
                  "Proceed To Pay",
                  style: TextStyle(
                      color: Colors.white,
                      fontSize: 18,
                      fontWeight: FontWeight.w500),
                ),
                onPressed: _selected == null
                    ? null
                    : () {
                        Navigator.of(context).pop(_selected);
                      },
              ),
            ),
          ],
        ),
      ),
    );
  }

  Function onSelected(PaymentMethod method) {
    setState(() {
      _selected = method;
    });
  }

  Widget buildWalletsList(List<PaymentWallet> wallets) {
    return buildGroup(
        title: "Wallets",
        description: "Free",
        onPressed: null,
        axis: Axis.horizontal,
        children: wallets
            .map((e) => WalletWidget(
                  wallet: e,
                  isSelected: _selected == e,
                  onPressed: (isSelected, wallet) {
                    onSelected(wallet);
                  },
                ))
            .toList());
  }

  Widget buildCardsList(List<PaymentCard> cards) {
    return buildGroup(
        title: "Credit Cards",
        description: "3% Free",
        action: "Add a New Credit Card",
        onPressed: _onAddCard,
        children: cards
            .map((e) => PaymentCardWidget(
                  card: e,
                  isSelected: _selected == e,
                  onSelected: (wallet, isSelected) {
                    onSelected(wallet);
                  },
                ))
            .toList());
  }

  Future<void> _onAddCard() async {
    PaymentCard paymentCard =
        await Navigator.of(context).push(MaterialPageRoute(
      builder: (context) => CardAddScreen(),
    ));
    if (paymentCard != null) {
      BlocProvider.of<PaymentMethodListBloc>(context)
          .add(PaymentMethodAddEvent(paymentCard));
    }
  }

  Widget buildBanksList(List<PaymentBank> banks) {
    return buildGroup(
        title: "Bank Accounts",
        description: "Free",
        action: "Add a New Bank Account",
        onPressed: _onAddBank,
        children: banks
            .map((e) => PaymentBankWidget(
                  bank: e,
                  isSelected: _selected == e,
                  onSelected: (wallet, isSelected) {
                    onSelected(wallet);
                  },
                ))
            .toList());
  }

  Future<void> _onAddBank() async {
    PaymentBank paymentBank =
        await Navigator.of(context).push(MaterialPageRoute(
      builder: (context) => BankAddScreen(),
    ));
    if (paymentBank != null) {
      BlocProvider.of<PaymentMethodListBloc>(context)
          .add(PaymentMethodAddEvent(paymentBank));
    }
  }

  Widget buildGroup(
      {String title,
      String description,
      String action,
      Function() onPressed,
      List<Widget> children,
      Axis axis = Axis.vertical}) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Padding(
          padding: const EdgeInsets.symmetric(vertical: 12),
          child: Row(
            children: <Widget>[
              Expanded(
                  child: Text(
                "$title",
                style: TextStyle(
                    color: Colors.black45, fontWeight: FontWeight.w700),
              )),
              CupertinoButton(
                  padding: EdgeInsets.all(0),
                  minSize: 0,
                  child: Row(
                    mainAxisSize: MainAxisSize.min,
                    children: [
                      Text(
                        "Add",
                        style: TextStyle(fontSize: 16),
                      ),
                      Icon(
                        Icons.add,
                        size: 22,
                      )
                    ],
                  ),
                  onPressed: onPressed)
            ],
          ),
        ),
        SizedBox(
          height: axis == Axis.horizontal ? 140 : null,
          child: ListView.separated(
            padding: EdgeInsets.symmetric(vertical: 12),
            separatorBuilder: (context, index) => Divider(),
            itemBuilder: (context, index) => children[index],
            shrinkWrap: true,
            itemCount: children.length,
            physics: ClampingScrollPhysics(),
            scrollDirection: axis,
          ),
        ),
      ],
    );
  }
}

class OutlineButtonBlueBorder extends StatelessWidget {
  const OutlineButtonBlueBorder({
    Key key,
    @required this.text,
    @required this.onPressed,
    @required this.fontSize,
    this.padding,
  }) : super(key: key);

  final String text;
  final Function onPressed;
  final double fontSize;
  final EdgeInsets padding;

  @override
  Widget build(BuildContext context) {
    return OutlineButton(
        padding: padding,
        borderSide: BorderSide(
          color: MetaColors.secondary,
        ),
        shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.all(Radius.circular(5))),
        child: Text(
          text,
          style: TextStyle(color: MetaColors.secondary, fontSize: fontSize),
        ),
        onPressed: onPressed);
  }
}
